package pogr;

import java.awt.Canvas;
import java.awt.Point;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import javax.swing.JLabel;

/**
 * Třída pro vyplňování geometricky zadaného polygonu pomocí rozkladových řádků
 * @author Kamil
 */
public class Scanline extends Algoritmus {

    private ArrayList<Line2D.Double> usecky;

    private int xmin,xmax,ymin,ymax;

    /**
     * Funkce generuje ArrayList usecky, obsahující všechny hrany polygonu. 
     * Hrany jsou na spodním okraji zkráceny o jeden pixel, abychom se vyhnuli 
     * lichému počtu průsečíků s rozkladovou řádkou
     */
    protected void zkratUsecky() {
        usecky.clear();
        
        Line2D.Double usecka;
        for (int i=0;i<polygon.size()-1;i++) {
            if (polygon.get(i).y<polygon.get(i+1).y) {
                usecka=new Line2D.Double(polygon.get(i).x,polygon.get(i).y,polygon.get(i+1).x,polygon.get(i+1).y-1);
            } else if (polygon.get(i).y>polygon.get(i+1).y) {
                usecka=new Line2D.Double(polygon.get(i+1).x,polygon.get(i+1).y,polygon.get(i).x,polygon.get(i).y-1);
            } else {
                usecka = new Line2D.Double(polygon.get(i), polygon.get(i+1));
            }
            usecky.add(usecka);
        }
    }

    private ArrayList<Point> setrid(ArrayList<Point> list) {
        boolean serazeno;
        do {
            serazeno=true;
            for (int i=0; i<list.size()-1;i++) {
                if (list.get(i).x>list.get(i+1).x) {
                    Point tmp = list.get(i);
                    list.set(i, list.get(i+1));
                    list.set(i+1, tmp);
                    serazeno=false;
                }
            }
        } while(serazeno==false);

        return list;
    }

    /**
     * Funkce heldá vechyn průsečíky na zadaném řádku
     * @param radek Řádek ,ve kterém se průsečeíky hledají
     * @param zac   x-ová souřadnice začátku řádku
     * @param kon   x-ová souřadnice konce řádku
     * @return      seznam nalezených průsečíků
     */
    protected ArrayList<Point> najdiPruseciky(int radek, int zac, int kon) {
        ArrayList<Point> pruseciky = new ArrayList<Point>();
        for(int i=0;i<polygon.size()-1;i++) {
            if ((polygon.get(i).y>=radek && polygon.get(i+1).y<radek) || (polygon.get(i).y<radek && polygon.get(i+1).y>=radek)) {
                double m = (double)(polygon.get(i).y-polygon.get(i+1).y)/(double)(polygon.get(i).x-polygon.get(i+1).x);
                double x = polygon.get(i).x + (1/m)*(radek-polygon.get(i).y);
                pruseciky.add(new Point( (int)Math.round(x), radek));
            }
        }

        if (pruseciky.size()==1) {

        }

        return setrid(pruseciky);
    }


    /**
     * Konstruktor
     * @param _platno   Plátno pro kreslení
     * @param label     Label pro zobrazování zpráv pro uživatele
     */
    public Scanline(MujCanvas _platno, JLabel label) {
        super(_platno, label);
        usecky = new ArrayList<Line2D.Double>();
        xmin=0;
        xmax=platno.getWidth();
        ymin=0;
        ymax=platno.getHeight();
    }

    /**
     * Funkce vyplňující polygon pomocí rozkladových řádků
     * @param postup Pokud je true, bude průběžně vykreslován postup vyplňování na obrazovku
     */
    @Override
    public void vyplnPolygon(boolean postup) {

        ArrayList<Point> pruseciky = new ArrayList<Point>();

        setBarva(0xFF0000);
        for (int i = ymin; i < ymax; i++) {
            pruseciky = najdiPruseciky(i, xmax, xmin);
            System.out.print("Pruseciku je "+pruseciky.size()+"\n");
            if (pruseciky.size()==1) {
                System.out.print("Ten jeden je "+pruseciky.get(0));
            }
            for (int j = 0; j < pruseciky.size(); j += 2) {
                vykresliUsecku(pruseciky.get(j).x, pruseciky.get(j).y, pruseciky.get(j + 1).x, pruseciky.get(j + 1).y);
                if (postup) {
                    //Vynucene vzkresleni
                    g.drawImage(obrazek, 0, 0, null);
                }
            }
        }
        //g.drawImage(obrazek, 0, 0, io);
        platno.setObrazek(obrazek);
        platno.repaint();
        zprava("Hotovo");
    }

}