package pogr;

import java.awt.Canvas;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JLabel;

/**
 *  Třída, která zastupuje algoritmus pro inverzní vyplňování geometricky zadaného polygonu  pomocí šablony.
 *
 * @author Kamil Ondrák
 */
public class Stencil extends Algoritmus {
    
    private int[][] sb; //Stencil buffer

    /**
     * Kostruktor
     *
     * @param _platno   plátno pro kreslení
     * @param label     label pro zobrazování zpráv uživateli
     */
    public Stencil(MujCanvas _platno, JLabel label) {
        super(_platno, label);
    }

    /**
     * Metoda provádějící vyplnění polygonu.
     *
     * Nejprve se sestaví 2D pole sb, zastupující stencil buffer, a inicializuje všechno jeho prvky na -1.
     * Poté metoda projde všechny hrany polygonu a oblast vpravo od nich ve stencil buffer vynásobí -1.
     * Na závěr jsou pak vybareveny body, které mají v sb hodnotu 1.
     *
     * @param postup Pokud je true, bude průběžně vykreslován postup vyplňování na obrazovku
     */
    @Override
    public void vyplnPolygon(boolean postup) {
//        System.out.print("Sirka je "+obrazek.getWidth());
        sb = new int[obrazek.getHeight()][obrazek.getWidth()];
        // nulovani sb
        for (int i=0;i<sb.length;i++) {
            for (int j=0;j<sb[0].length;j++) {
                sb[i][j]=-1;
            }
        }
        //projiti vsech hran
        for (int i =0; i<polygon.size()-1;i++) {
            //zjisteni orientace zkoumane hrany
            int zac,kon;
            if (polygon.get(i).y>polygon.get(i+1).y) {
                zac=polygon.get(i+1).y;
                kon=polygon.get(i).y;
            }
            else if (polygon.get(i).y<polygon.get(i+1).y) {
                zac=polygon.get(i).y;
                kon=polygon.get(i+1).y;
            }
            else {
                continue;
            }
            //projiti cele hrany a negace bodu vpravo od ni
            for (int j=zac; j<kon;j++) {
                double m = (double) (polygon.get(i).y - polygon.get(i + 1).y) / (double) (polygon.get(i).x - polygon.get(i + 1).x);
                double x = polygon.get(i).x + (1 / m) * (j - polygon.get(i).y);
                for (int k=(int)x;k<sb[j].length;k++) {
                    sb[j][k]*=-1;
                }
            }

            if (postup) {
                setBarva(0xFF0000);
                for (int k = 0; k < sb.length; k++) {
                    for (int j = 0; j < sb[0].length; j++) {
                        if (sb[k][j] == 1) {
                            obrazek.setRGB(j, k, barva);
                        }
                        else {
                            obrazek.setRGB(j, k, 0xFFFFFF);
                        }
                    }
                }
                //Vynucene vzkresleni
                g.drawImage(obrazek, 0, 0, null);
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException ex) {
                    Logger.getLogger(Stencil.class.getName()).log(Level.SEVERE, null, ex);
                }
                platno.repaint();
            }

        }
        //vykresleni obsahu sb na obrazovku
        setBarva(0xFF0000);
        for (int i = 0; i < sb.length; i++) {
            for (int j = 0; j < sb[0].length; j++) {
                if (sb[i][j] == 1) {
                    obrazek.setRGB(j, i, barva);
                }
                else {
                    obrazek.setRGB(j, i, 0xFFFFFF);
                }
            }

        }
        //g.drawImage(obrazek, 0, 0, io);
        platno.setObrazek(obrazek);
        platno.repaint();
        zprava("Hotovo");

    }
}
